gleamgen/render/config

Types

pub type Config {
  Config(
    render_function_parameter_types: Bool,
    combine_equivalent_branches: Bool,
    auto_fix_parameters: Bool,
    inline_instantly_called_anonymous_functions: Bool,
    annotate_type_in_let_declarations: Bool,
  )
}

Constructors

  • Config(
      render_function_parameter_types: Bool,
      combine_equivalent_branches: Bool,
      auto_fix_parameters: Bool,
      inline_instantly_called_anonymous_functions: Bool,
      annotate_type_in_let_declarations: Bool,
    )

    Arguments

    render_function_parameter_types

    Include explicit type annotations for function parameters when possible.

    combine_equivalent_branches

    Merge case branches that produce equivalent outputs.

    auto_fix_parameters

    Automatically add missing parameters to function definitions and calls

    inline_instantly_called_anonymous_functions

    Simplify expressions where an anonymous function is immediately called.

    annotate_type_in_let_declarations

    Include explicit type annotations on let bindings.

Values

pub const default_config: Config

Recommended defaults for rendering.

Search Document